@@ -168,6 +168,9 @@ pub use crate::xdg::{
     NotificationHandle,
     dbus_stack,
     DbusStack,
+    ActionResponse,
+    CloseHandler,
+    CloseReason,
 };
 
 #[cfg(all(feature = "server", unix, not(target_os = "macos")))]
